@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <errno.h>
 #include <string.h>
@@ -121,20 +101,45 @@ static int error_resource(void *object, void *data)
 {
 	struct pw_resource *r = object;
 	struct error_data *d = data;
-	if (r && r->bound_id == d->id)
+
+	if (r && r->bound_id == d->id) {
+		pw_log_debug("%p: client error for global %u: %d (%s)",
+				r, d->id, d->res, d->error);
 		pw_resource_error(r, d->res, d->error);
+	}
 	return 0;
 }
 
 static int client_error(void *object, uint32_t id, int res, const char *error)
 {
 	struct resource_data *data = object;
+	struct pw_resource *resource = data->resource;
+	struct pw_impl_client *sender = resource->client;
 	struct pw_impl_client *client = data->client;
 	struct error_data d = { id, res, error };
+	struct pw_global *global;
 
-	pw_log_debug("%p: error for global %d", client, id);
+	/* Check the global id provided by sender refers to a registered global
+	 * known to the sender.
+	 */
+	if ((global = pw_context_find_global(resource->context, id)) == NULL)
+		goto error_no_id;
+	if (sender->recv_generation != 0 && global->generation > sender->recv_generation)
+		goto error_stale_id;
+
+	pw_log_debug("%p: sender %p: error for global %u", client, sender, id);
 	pw_map_for_each(&client->objects, error_resource, &d);
 	return 0;
+
+error_no_id:
+	pw_log_debug("%p: sender %p: error for invalid global %u", client, sender, id);
+	pw_resource_errorf(resource, -ENOENT, "no global %u", id);
+	return -ENOENT;
+error_stale_id:
+	pw_log_debug("%p: sender %p: error for stale global %u generation:%"PRIu64" recv-generation:%"PRIu64,
+			client, sender, id, global->generation, sender->recv_generation);
+	pw_resource_errorf(resource, -ESTALE, "no global %u any more", id);
+	return -ESTALE;
 }
 
 static bool has_key(const char * const keys[], const char *key)
